@@ -1,10 +1,15 @@
 from dropbox.client import DropboxClient
 from dropbox.session import DropboxSession
 from flask import request, session as flask_session, url_for
-from oauth.oauth import OAuthToken
 from werkzeug.routing import BuildError as RoutingBuildError
 from werkzeug.utils import cached_property
 
+try:
+    # For Dropbox SDK 1.4.1+
+    from dropbox.session import OAuthToken
+except ImportError:
+    from oauth.oauth import OAuthToken
+
 from .settings import DROPBOX_ACCESS_TOKEN_KEY, DROPBOX_REQUEST_TOKEN_KEY
 
 
@@ -77,8 +82,8 @@ class Dropbox(object):
         """
         Check if current user logged in with Dropbox or not.
         """
-        return DROPBOX_ACCESS_TOKEN_KEY in flask_session and \
-               isinstance(flask_session[DROPBOX_ACCESS_TOKEN_KEY], OAuthToken)
+        db_session_key = flask_session.get(DROPBOX_ACCESS_TOKEN_KEY)
+        return db_session_key and isinstance(db_session_key, OAuthToken)
 
     def login(self, request_token):
         """
@@ -145,6 +150,20 @@ class Dropbox(object):
                               self.DROPBOX_ACCESS_TYPE)
 
 
+# Monkey-patch Dropbox SDK ``OAuthToken`` class to add support of pickling
+# tokens. Without this monkey-patch all tries to save any token in Flask
+# session will cause "TypeError: a class that defines __slots__ without
+# defining __getstate__ cannot be pickled"
+if hasattr(OAuthToken, '__slots__'):
+    OAuthToken.__getstate__ = \
+        lambda instance: dict([(key, getattr(instance, key)) \
+                                for key in instance.__slots__])
+    OAuthToken.__setstate__ = \
+        lambda instance, data: [setattr(instance, key, value) \
+                                for key, value in data.iteritems() \
+                                if key in instance.__slots__]
+
+
 def safe_url_for(url, *args, **kwargs):
     """
     Fail silently ``BuildError`` on calling original Flask's ``url_for``
